@@ -1,9 +1,20 @@
 /*
  * armor.h - Routines to (de)armor OpenPGP packet streams.
  *
- * Jonathan McDowell <noodles@earth.li>
+ * Copyright 2002-2004, 2011 Jonathan McDowell <noodles@earth.li>
  *
- * Copyright 2002 Project Purple
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc., 51
+ * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __ARMOR_H__
@@ -21,7 +32,7 @@
  *	using putchar_func.
  */
 int armor_openpgp_stream(int (*putchar_func)(void *ctx, size_t count,
-						unsigned char *c),
+						void *c),
 				void *ctx,
 				struct openpgp_packet_list *packets);
 
@@ -36,7 +47,7 @@ int armor_openpgp_stream(int (*putchar_func)(void *ctx, size_t count,
  *	packets.
  */
 int dearmor_openpgp_stream(int (*getchar_func)(void *ctx, size_t count,
-					unsigned char *c),
+					void *c),
 				void *ctx,
 				struct openpgp_packet_list **packets);
 
