@@ -1,9 +1,20 @@
 /*
  * lookup.c - CGI to lookup keys.
  *
- * Jonathan McDowell <noodles@earth.li>
+ * Copyright 2002-2005,2007-2009,2011 Jonathan McDowell <noodles@earth.li>
  *
- * Copyright 2002 Project Purple
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc., 51
+ * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <inttypes.h>
@@ -19,6 +30,7 @@
 #include "cleanup.h"
 #include "getcgi.h"
 #include "keydb.h"
+#include "keyid.h"
 #include "keyindex.h"
 #include "log.h"
 #include "mem.h"
@@ -32,9 +44,11 @@
 #define OP_INDEX   2
 #define OP_VINDEX  3
 #define OP_PHOTO   4
+#define OP_HGET    5
 
 void find_keys(char *search, uint64_t keyid, bool ishex,
-		bool fingerprint, bool exact, bool verbose, bool mrhkp)
+		bool fingerprint, bool skshash, bool exact, bool verbose,
+		bool mrhkp)
 {
 	struct openpgp_publickey *publickey = NULL;
 	int count = 0;
@@ -49,7 +63,8 @@ void find_keys(char *search, uint64_t keyid, bool ishex,
 			printf("info:1:%d\n", count);
 			mrkey_index(publickey);
 		} else {
-			key_index(publickey, verbose, fingerprint, true);
+			key_index(publickey, verbose, fingerprint, skshash,
+				true);
 		}
 		free_publickey(publickey);
 	} else if (count == 0) {
@@ -78,6 +93,7 @@ int main(int argc, char *argv[])
 	int i;
 	int indx = 0;
 	bool fingerprint = false;
+	bool skshash = false;
 	bool exact = false;
 	bool ishex = false;
 	bool mrhkp = false;
@@ -88,12 +104,15 @@ int main(int argc, char *argv[])
 	struct openpgp_packet_list *packets = NULL;
 	struct openpgp_packet_list *list_end = NULL;
 	int result;
+	struct skshash hash;
 
 	params = getcgivars(argc, argv);
 	for (i = 0; params != NULL && params[i] != NULL; i += 2) {
 		if (!strcmp(params[i], "op")) {
 			if (!strcmp(params[i+1], "get")) {
 				op = OP_GET;
+			} else if (!strcmp(params[i+1], "hget")) {
+				op = OP_HGET;
 			} else if (!strcmp(params[i+1], "index")) {
 				op = OP_INDEX;
 			} else if (!strcmp(params[i+1], "vindex")) {
@@ -104,7 +123,17 @@ int main(int argc, char *argv[])
 		} else if (!strcmp(params[i], "search")) {
 			search = params[i+1];
 			params[i+1] = NULL;
-			if (search != NULL) {
+			if (search != NULL && strlen(search) == 42 &&
+					search[0] == '0' && search[1] == 'x') {
+				/*
+				 * Fingerprint. Truncate to last 64 bits for
+				 * now.
+				 */
+				keyid = strtoull(&search[26], &end, 16);
+				if (end != NULL && *end == 0) {
+					ishex = true;
+				}
+			} else if (search != NULL) {
 				keyid = strtoull(search, &end, 16);
 				if (*search != 0 &&
 						end != NULL &&
@@ -118,6 +147,10 @@ int main(int argc, char *argv[])
 			if (!strcmp(params[i+1], "on")) {
 				fingerprint = true;
 			}
+		} else if (!strcmp(params[i], "hash")) {
+			if (!strcmp(params[i+1], "on")) {
+				skshash = true;
+			}
 		} else if (!strcmp(params[i], "exact")) {
 			if (!strcmp(params[i+1], "on")) {
 				exact = true;
@@ -163,7 +196,12 @@ int main(int argc, char *argv[])
 		config.dbbackend->initdb(false);
 		switch (op) {
 		case OP_GET:
-			if (ishex) {
+		case OP_HGET:
+			if (op == OP_HGET) {
+				parse_skshash(search, &hash);
+				result = config.dbbackend->fetch_key_skshash(
+					&hash, &publickey);
+			} else if (ishex) {
 				result = config.dbbackend->fetch_key(keyid,
 					&publickey, false);
 			} else {
@@ -193,12 +231,12 @@ int main(int argc, char *argv[])
 			}
 			break;
 		case OP_INDEX:
-			find_keys(search, keyid, ishex, fingerprint, exact,
-					false, mrhkp);
+			find_keys(search, keyid, ishex, fingerprint, skshash,
+					exact, false, mrhkp);
 			break;
 		case OP_VINDEX:
-			find_keys(search, keyid, ishex, fingerprint, exact,
-					true, mrhkp);
+			find_keys(search, keyid, ishex, fingerprint, skshash,
+					exact, true, mrhkp);
 			break;
 		case OP_PHOTO:
 			if (config.dbbackend->fetch_key(keyid, &publickey,
@@ -226,8 +264,7 @@ int main(int argc, char *argv[])
 	}
 	if (!mrhkp) {
 		puts("<hr>");
-		puts("Produced by onak " ONAK_VERSION 
-				" by Jonathan McDowell");
+		puts("Produced by onak " ONAK_VERSION );
 		end_html();
 	}
 
