@@ -1,9 +1,20 @@
 /*
  * parsekey.c - Routines to parse an OpenPGP key.
  *
- * Jonathan McDowell <noodles@earth.li>
+ * Copyright 2002-2004,2007-2008,2011 Jonathan McDowell <noodles@earth.li>
  *
- * Copyright 2002 Project Purple
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc., 51
+ * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdbool.h>
@@ -16,6 +27,7 @@
 #include "ll.h"
 #include "log.h"
 #include "mem.h"
+#include "openpgp.h"
 #include "parsekey.h"
 
 /**
@@ -45,7 +57,7 @@ int parse_keys(struct openpgp_packet_list *packets,
 
 	while (packets != NULL) {
 		switch (packets->packet->tag) {
-		case 2:
+		case OPENPGP_PACKET_SIGNATURE:
 			/*
 			 * It's a signature packet. Add it to either the public
 			 * key, to the current UID or the current subkey.
@@ -68,13 +80,15 @@ int parse_keys(struct openpgp_packet_list *packets,
 				 * if it's a revocation.
 				 */
 				if (packets->packet->data[0] == 3 &&
-					packets->packet->data[2] == 0x20) {
+					packets->packet->data[2] ==
+						OPENPGP_SIGTYPE_KEY_REV) {
 					/*
 					 * Type 3 key, 0x20 == revocation
 					 */
 					curkey->revoked = true;
 				} else if (packets->packet->data[0] == 4 &&
-					packets->packet->data[1] == 0x20) {
+					packets->packet->data[1] ==
+						OPENPGP_SIGTYPE_KEY_REV) {
 					/*
 					 * Type 4 key, 0x20 == revocation
 					 */
@@ -82,7 +96,7 @@ int parse_keys(struct openpgp_packet_list *packets,
 				}
 			}
 			break;
-		case 6:
+		case OPENPGP_PACKET_PUBLICKEY:
 			/*
 			 * It's a public key packet, so start a new key in our
 			 * list.
@@ -98,8 +112,8 @@ int parse_keys(struct openpgp_packet_list *packets,
 			curkey->publickey = packet_dup(packets->packet);
 			count++;
 			break;
-		case 13:
-		case 17:
+		case OPENPGP_PACKET_UID:
+		case OPENPGP_PACKET_UAT:
 			/*
 			 * It's a UID packet (or a photo id, which is similar).
 			 */
@@ -109,7 +123,7 @@ int parse_keys(struct openpgp_packet_list *packets,
 				uid,
 				packet_dup(packets->packet));
 			break;
-		case 14:
+		case OPENPGP_PACKET_PUBLICSUBKEY:
 			/*
 			 * It's a subkey packet.
 			 */
@@ -118,7 +132,7 @@ int parse_keys(struct openpgp_packet_list *packets,
 				subkey,
 				packet_dup(packets->packet));
 			break;
-		case 12:
+		case OPENPGP_PACKET_TRUST:
 		case 61:
 			/*
 			 * One of:
@@ -168,7 +182,7 @@ int debug_packet(struct openpgp_packet *packet)
  *	ready for parsing as a public key or whatever.
  */
 int read_openpgp_stream(int (*getchar_func)(void *ctx, size_t count,
-				unsigned char *c),
+				void *c),
 				void *ctx,
 				struct openpgp_packet_list **packets,
 				int maxnum)
@@ -284,7 +298,8 @@ int read_openpgp_stream(int (*getchar_func)(void *ctx, size_t count,
 			}
 
 			if (rc == 0) {
-				if (curpacket->packet->tag == 6) {
+				if (curpacket->packet->tag ==
+						OPENPGP_PACKET_PUBLICKEY) {
 					keys++;
 				}
 				curpacket->packet->data =
@@ -322,7 +337,7 @@ int read_openpgp_stream(int (*getchar_func)(void *ctx, size_t count,
  *	packet stream from a linked list of packets.
  */
 int write_openpgp_stream(int (*putchar_func)(void *ctx, size_t count,
-						unsigned char *c),
+						void *c),
 				void *ctx,
 				struct openpgp_packet_list *packets)
 {
